@@ -18,7 +18,7 @@ import net.argius.stew.io.*;
 /**
  * ʃZbgp̃e[uB
  */
-class ResultSetTable extends JTable {
+class ResultSetTable extends JTable implements TextSearch {
 
     private static final Logger log = LoggerFactory.getLogger(ResultSetTable.class);
 
@@ -31,6 +31,12 @@ class ResultSetTable extends JTable {
     private String columnAdjustMode;
     private int dragStartColumn;
 
+    /*
+     * ZҏWJneditCellAt(int, int, EventObject)Ă΂ꂽꍇA
+     * KeyEventߑłȂ߁AOKeyEventۑĎgB
+     */
+    private volatile KeyEvent lastKeyEvent;
+
     /**
      * ResultSetTable̐B
      * @param env s
@@ -107,6 +113,29 @@ class ResultSetTable extends JTable {
             }
 
         });
+        addKeyListener(new KeyListener() {
+
+            public void keyTyped(KeyEvent e) {
+                setLastKeyEvent(e);
+            }
+
+            public void keyReleased(KeyEvent e) {
+                setLastKeyEvent(e);
+            }
+
+            public void keyPressed(KeyEvent e) {
+                setLastKeyEvent(e);
+            }
+
+        });
+    }
+
+    /**
+     * lastKeyEvent̐ݒB
+     * @param lastKeyEvent lastKeyEvent
+     */
+    void setLastKeyEvent(KeyEvent lastKeyEvent) {
+        this.lastKeyEvent = lastKeyEvent;
     }
 
     /* (overridden)
@@ -192,6 +221,81 @@ class ResultSetTable extends JTable {
         }
     }
 
+    /* (overridden)
+     * @see javax.swing.JTable#editCellAt(int, int, java.util.EventObject)
+     */
+    public boolean editCellAt(int row, int column, EventObject e) {
+        boolean succeeded = super.editCellAt(row, column, e);
+        if (succeeded) {
+            if (editorComp instanceof JTextField) {
+                // #208: ZҏW[hŒlIԂɂ
+                if (lastKeyEvent != null && lastKeyEvent.getKeyCode() != KeyEvent.VK_F2) {
+                    JTextField editor = (JTextField)editorComp;
+                    editor.selectAll();
+                }
+            }
+        }
+        return succeeded;
+    }
+
+    /* (overridden)
+     * @see net.argius.stew.gui.TextSearch#search(net.argius.stew.gui.TextSearchMatcher)
+     */
+    public boolean search(TextSearchMatcher matcher) {
+        final int rowCount = getRowCount();
+        if (rowCount <= 0) {
+            return false;
+        }
+        final int columnCount = getColumnCount();
+        final boolean backward = matcher.isBackward();
+        final int amount = backward ? -1 : 1;
+        final int rowStart = backward ? rowCount - 1 : 0;
+        final int rowEnd = backward ? 0 : rowCount - 1;
+        final int columnStart = backward ? columnCount - 1 : 0;
+        final int columnEnd = backward ? 0 : columnCount - 1;
+        int row = rowStart;
+        int column = columnStart;
+        if (getSelectedColumnCount() > 0) {
+            column = getSelectedColumn();
+            row = getSelectedRow() + amount;
+            if (backward) {
+                if (row < 0) {
+                    --column;
+                    if (column < 0) {
+                        return false;
+                    }
+                    row = rowStart;
+                }
+            } else {
+                if (row >= rowCount) {
+                    ++column;
+                    if (column >= columnCount) {
+                        return false;
+                    }
+                    row = rowStart;
+                }
+            }
+        }
+        for (; backward ? column >= columnEnd : column <= columnEnd; column += amount) {
+            for (; backward ? row >= rowEnd : row <= rowEnd; row += amount) {
+                if (matcher.find(String.valueOf(getValueAt(row, column)))) {
+                    changeSelection(row, column, false, false);
+                    requestFocus();
+                    return true;
+                }
+            }
+            row = rowStart;
+        }
+        return false;
+    }
+
+    /* (overridden)
+     * @see net.argius.stew.gui.TextSearch#reset()
+     */
+    public void reset() {
+        // empty
+    }
+
     /**
      * TCY{̎擾B
      * @param font ƂȂtHg
@@ -256,6 +360,7 @@ class ResultSetTable extends JTable {
                     return;
                 }
                 selectColumn(index0, index1);
+                requestFocus();
             }
         }
     }
@@ -300,13 +405,12 @@ class ResultSetTable extends JTable {
 
     /**
      * \[gs˗B
-     * @param clickedIndex Iꂽ
+     * @param selectedIndex Iꂽ
      */
-    void requestSort(int clickedIndex) {
-        if (clickedIndex > 0) {
-            final int threshold = 2000;
-            int rate = getRowCount() * (1 + getColumnCount() / 10);
-            if (rate >= threshold) {
+    void requestSort(int selectedIndex) {
+        if (selectedIndex > 0) {
+            final int threshold = 100000;
+            if (getRowCount() > threshold) {
                 final String message = Messages.getString("window.ResultSetTable.sort.confirm");
                 int result = DialogMessage.confirmYesNo(getParent(), message);
                 if (result != JOptionPane.OK_OPTION) {
@@ -314,47 +418,30 @@ class ResultSetTable extends JTable {
                 }
             }
         }
-        sort(clickedIndex);
+        sort(selectedIndex);
     }
 
     /**
      * \[gB
-     * }\[gASYgpB
-     * @param clickedIndex Iꂽ
+     * @param index Iꂽ
      */
-    private void sort(int clickedIndex) {
-        int rows = getRowCount();
-        int cols = getColumnCount();
-        boolean sortedAsc = sortedAscArray[clickedIndex];
-        Comparator cmp = new ResultSetValueComparator();
-        for (int i = 1; i < rows; i++) {
-            for (int j = i; j > 0; j--) {
-                Object v1 = getValueAt(j - 1, clickedIndex);
-                Object v2 = getValueAt(j, clickedIndex);
-                if (cmp.compare(v1, v2) > 0 == sortedAsc) {
-                    break;
-                }
-                swap(j, j - 1);
-            }
-        }
-        for (int i = 0; i < cols; i++) {
-            sortedAscArray[i] = false;
-        }
-        sortedAscArray[clickedIndex] = !sortedAsc;
-    }
+    private void sort(final int index) {
+        final boolean sortedAsc = sortedAscArray[index];
+        final Comparator cmp = new ResultSetValueComparator();
+        final int flip = sortedAsc ? -1 : 1;
+        Collections.sort(((DefaultTableModel)getModel()).getDataVector(), new Comparator() {
 
-    /**
-     * lB
-     * @param rowIndex1 sCfbNX1
-     * @param rowIndex2 sCfbNX1
-     */
-    private void swap(int rowIndex1, int rowIndex2) {
-        int cols = getColumnCount();
-        for (int i = 0; i < cols; i++) {
-            Object o = getValueAt(rowIndex1, i);
-            setValueAt(getValueAt(rowIndex2, i), rowIndex1, i);
-            setValueAt(o, rowIndex2, i);
+            public int compare(Object o1, Object o2) {
+                List a1 = (List)o1;
+                List a2 = (List)o2;
+                return cmp.compare(a1.get(index), a2.get(index)) * flip;
+            }
+
+        });
+        for (int i = 0; i < sortedAscArray.length; i++) {
+            sortedAscArray[i] = index == i && !sortedAsc;
         }
+        repaint();
     }
 
     /**
@@ -377,62 +464,69 @@ class ResultSetTable extends JTable {
         if (columnAdjustMode == null) {
             return;
         }
+        final boolean byHeader;
+        final boolean byValue;
         if (columnAdjustMode.equalsIgnoreCase("header")) {
-            // wb_̕ɂ钲
-            int fontSize = getFont().getSize();
-            TableColumnModel columnModel = getColumnModel();
-            for (int columnIndex = 0, n = getColumnCount(); columnIndex < n; columnIndex++) {
-                TableColumn column = columnModel.getColumn(columnIndex);
-                String headerName = String.valueOf(column.getHeaderValue());
-                int size = (headerName.length() + 1) * fontSize;
-                column.setPreferredWidth(size);
-            }
+            byHeader = true;
+            byValue = false;
         } else if (columnAdjustMode.equalsIgnoreCase("value")) {
-            // l̕ɂ钲
-            int rowCount = getRowCount();
-            if (rowCount == 0) {
-                return;
-            }
-            int fontSize = getFont().getSize();
-            TableColumnModel columnModel = getColumnModel();
-            JTableHeader header = getTableHeader();
-            for (int columnIndex = 0, n = getColumnCount(); columnIndex < n; columnIndex++) {
+            byHeader = false;
+            byValue = true;
+        } else if (columnAdjustMode.equalsIgnoreCase("header-and-value")) {
+            byHeader = true;
+            byValue = true;
+        } else {
+            byHeader = false;
+            byValue = false;
+        }
+        int rowCount = getRowCount();
+        if (!byHeader && byValue && rowCount == 0) {
+            return;
+        }
+        final float max = getParent().getWidth() * 0.8f;
+        TableColumnModel columnModel = getColumnModel();
+        JTableHeader header = getTableHeader();
+        for (int columnIndex = 0, n = getColumnCount(); columnIndex < n; columnIndex++) {
+            float size = 0f;
+            if (byHeader) {
+                // wb_̕ɂ钲
                 TableColumn column = columnModel.getColumn(columnIndex);
-                TableCellRenderer headerRenderer = column.getHeaderRenderer();
-                if (headerRenderer == null) {
-                    headerRenderer = header.getDefaultRenderer();
+                TableCellRenderer renderer = column.getHeaderRenderer();
+                if (renderer == null) {
+                    renderer = header.getDefaultRenderer();
                 }
-                int size = 1;
-                if (headerRenderer != null) {
-                    size = headerRenderer.getTableCellRendererComponent(this,
-                                                                        getValueAt(0, columnIndex),
-                                                                        false,
-                                                                        false,
-                                                                        0,
-                                                                        columnIndex)
-                                         .getPreferredSize().width;
+                if (renderer != null) {
+                    Component c = renderer.getTableCellRendererComponent(this,
+                                                                         column.getHeaderValue(),
+                                                                         false,
+                                                                         false,
+                                                                         0,
+                                                                         columnIndex);
+                    size = c.getPreferredSize().width * 1.5f;
                 }
+            }
+            if (byValue) {
+                // l̕ɂ钲
                 for (int rowIndex = 0; rowIndex < rowCount; rowIndex++) {
                     TableCellRenderer renderer = getCellRenderer(rowIndex, columnIndex);
-                    Object value = getValueAt(rowIndex, columnIndex);
-                    Component component = renderer.getTableCellRendererComponent(this,
-                                                                                 value,
-                                                                                 false,
-                                                                                 false,
-                                                                                 rowIndex,
-                                                                                 columnIndex);
-                    size = Math.max(size, component.getPreferredSize().width / 2);
-                    if (value instanceof java.util.Date) {
-                        size *= 2;
-                        if (log.isDebugEnabled()) {
-                            log.debug("[" + columnIndex + "] size = " + size);
-                        }
+                    if (renderer == null) {
+                        continue;
+                    }
+                    String value = String.valueOf(getValueAt(rowIndex, columnIndex));
+                    Component c = renderer.getTableCellRendererComponent(this,
+                                                                         value,
+                                                                         false,
+                                                                         false,
+                                                                         rowIndex,
+                                                                         columnIndex);
+                    size = Math.max(size, c.getPreferredSize().width);
+                    if (size >= max) {
                         break;
                     }
                 }
-                size = (int)(size * 1D / fontSize * 14);
-                columnModel.getColumn(columnIndex).setPreferredWidth(size);
             }
+            int width = Math.round(size > max ? max : size) + 1;
+            columnModel.getColumn(columnIndex).setPreferredWidth(width);
         }
     }
 
