@@ -9,6 +9,7 @@ import java.util.*;
 import java.util.List;
 
 import javax.swing.*;
+import javax.swing.table.*;
 
 import net.argius.logging.*;
 import net.argius.stew.io.*;
@@ -18,6 +19,66 @@ import net.argius.stew.io.*;
  */
 final class ResultSetTableAction {
 
+    /**
+     * <code>JUMP_TO_HOME</code>
+     */
+    static final String JUMP_TO_HOME = "jump-to-home";
+
+    /**
+     * <code>JUMP_TO_END</code>
+     */
+    static final String JUMP_TO_END = "jump-to-end";
+
+    /**
+     * <code>JUMP_TO_TOP</code>
+     */
+    static final String JUMP_TO_TOP = "jump-to-top";
+
+    /**
+     * <code>JUMP_TO_BOTTOM</code>
+     */
+    static final String JUMP_TO_BOTTOM = "jump-to-bottom";
+
+    /**
+     * <code>JUMP_TO_LEFTMOST</code>
+     */
+    static final String JUMP_TO_LEFTMOST = "jump-to-leftmost";
+
+    /**
+     * <code>JUMP_TO_RIGHTMOST</code>
+     */
+    static final String JUMP_TO_RIGHTMOST = "jump-to-rightmost";
+
+    /**
+     * <code>SELECT_TO_HOME</code>
+     */
+    static final String SELECT_TO_HOME = "select-to-home";
+
+    /**
+     * <code>SELECT_TO_END</code>
+     */
+    static final String SELECT_TO_END = "select-to-end";
+
+    /**
+     * <code>SELECT_TO_TOP</code>
+     */
+    static final String SELECT_TO_TOP = "select-to-top";
+
+    /**
+     * <code>SELECT_TO_BOTTOM</code>
+     */
+    static final String SELECT_TO_BOTTOM = "select-to-bottom";
+
+    /**
+     * <code>SELECT_TO_LEFTMOST</code>
+     */
+    static final String SELECT_TO_LEFTMOST = "select-to-leftmost";
+
+    /**
+     * <code>SELECT_TO_RIGHTMOST</code>
+     */
+    static final String SELECT_TO_RIGHTMOST = "select-to-rightmost";
+
     /**
      * <code>ADJUST_COLUMN_SIZE</code>
      */
@@ -28,6 +89,11 @@ final class ResultSetTableAction {
      */
     static final String COPY_CELL_WITH_ESCAPE = "copy-cell-with-escape";
 
+    /**
+     * <code>COPY_COLUMN_NAMES</code>
+     */
+    static final String COPY_COLUMN_NAMES = "copy-column-names";
+
     /**
      * <code>PASTE_TO_SELECTED_CELLS</code>
      */
@@ -88,8 +154,21 @@ final class ResultSetTableAction {
      */
     static void bindAction(final ResultSetTable table) {
         ResultSetTableAction a = new ResultSetTableAction(table);
+        a.bindJumpTo(JUMP_TO_HOME, KeyEvent.VK_HOME, false);
+        a.bindJumpTo(JUMP_TO_END, KeyEvent.VK_END, false);
+        a.bindJumpTo(JUMP_TO_TOP, KeyEvent.VK_UP, false);
+        a.bindJumpTo(JUMP_TO_BOTTOM, KeyEvent.VK_DOWN, false);
+        a.bindJumpTo(JUMP_TO_LEFTMOST, KeyEvent.VK_LEFT, false);
+        a.bindJumpTo(JUMP_TO_RIGHTMOST, KeyEvent.VK_RIGHT, false);
+        a.bindJumpTo(SELECT_TO_HOME, KeyEvent.VK_HOME, true);
+        a.bindJumpTo(SELECT_TO_END, KeyEvent.VK_END, true);
+        a.bindJumpTo(SELECT_TO_TOP, KeyEvent.VK_UP, true);
+        a.bindJumpTo(SELECT_TO_BOTTOM, KeyEvent.VK_DOWN, true);
+        a.bindJumpTo(SELECT_TO_LEFTMOST, KeyEvent.VK_LEFT, true);
+        a.bindJumpTo(SELECT_TO_RIGHTMOST, KeyEvent.VK_RIGHT, true);
         a.bindAdjustColumnSize();
         a.bindCopyCellWithEscape();
+        a.bindCopyColumnNames();
         a.bindPasteToSelectedCells();
         a.bindSetNullToSelectedCells();
         a.bindAddNewRow();
@@ -99,6 +178,121 @@ final class ResultSetTableAction {
         a.bindDeleteSelectedRows();
     }
 
+    /**
+     * JUMP_TO_̊蓖āB
+     * @param name ANV
+     * @param key ăL[ 
+     * @param withSelect I[h
+     */
+    private void bindJumpTo(String name, int key, boolean withSelect) {
+        final boolean extend = withSelect;
+        final CellCursor c = new CellCursor(table);
+        final Action action;
+        if (name.endsWith("-to-home")) {
+            action = new AbstractAction() {
+
+                public void actionPerformed(ActionEvent e) {
+                    table.changeSelection(0, 0, false, extend);
+                }
+
+            };
+        } else if (name.endsWith("-to-end")) {
+            action = new AbstractAction() {
+
+                public void actionPerformed(ActionEvent e) {
+                    int rowCount = table.getRowCount();
+                    int columnCount = table.getColumnCount();
+                    table.changeSelection(rowCount - 1, columnCount - 1, false, extend);
+                }
+
+            };
+        } else if (name.endsWith("-to-top")) {
+            action = new AbstractAction() {
+
+                public void actionPerformed(ActionEvent e) {
+                    table.changeSelection(0, c.getColumnPosition(), false, extend);
+                }
+
+            };
+        } else if (name.endsWith("-to-bottom")) {
+            action = new AbstractAction() {
+
+                public void actionPerformed(ActionEvent e) {
+                    int rowCount = table.getRowCount();
+                    table.changeSelection(rowCount - 1, c.getColumnPosition(), false, extend);
+                }
+
+            };
+        } else if (name.endsWith("-to-leftmost")) {
+            action = new AbstractAction() {
+
+                public void actionPerformed(ActionEvent e) {
+                    table.changeSelection(c.getRowPosition(), 0, false, extend);
+                }
+
+            };
+        } else if (name.endsWith("-to-rightmost")) {
+            action = new AbstractAction() {
+
+                public void actionPerformed(ActionEvent e) {
+                    int columnCount = table.getColumnCount();
+                    table.changeSelection(c.getRowPosition(), columnCount - 1, false, extend);
+                }
+
+            };
+        } else {
+            log.warn("unknown key: " + name);
+            return;
+        }
+        amap.put(name, action);
+        final int modifiers = InputEvent.CTRL_DOWN_MASK
+                              | (withSelect ? InputEvent.SHIFT_DOWN_MASK : 0);
+        imap.put(KeyStroke.getKeyStroke(key, modifiers), name);
+    }
+
+    /**
+     * I͈͂̍Ō̃Zɒu鑾gB
+     */
+    private static final class CellCursor {
+
+        private JTable table;
+
+        /**
+         * CellCursor̐B
+         * @param table
+         */
+        CellCursor(JTable table) {
+            this.table = table;
+        }
+
+        /**
+         * ʒu̎擾B
+         * @return ʒu
+         */
+        int getColumnPosition() {
+            int[] a = table.getSelectedColumns();
+            if (a == null || a.length == 0) {
+                return -1;
+            }
+            ListSelectionModel csm = table.getColumnModel().getSelectionModel();
+            return (a[0] == csm.getAnchorSelectionIndex()) ? a[a.length - 1] : a[0];
+        }
+
+        /**
+         * sʒu̎擾B
+         * @return sʒu
+         */
+        int getRowPosition() {
+            int[] a = table.getSelectedRows();
+            if (a == null || a.length == 0) {
+                return -1;
+            }
+            ListSelectionModel rsm = table.getSelectionModel();
+            return (a[0] == rsm.getAnchorSelectionIndex()) ? a[a.length - 1] : a[0];
+        }
+
+    }
+
     /**
      * ADJUST_COLUMN_SIZE̊蓖āB
      */
@@ -115,7 +309,7 @@ final class ResultSetTableAction {
     }
 
     /**
-     * CopyCellWithEscape̊蓖āB
+     * COPY_CELL_WITH_ESCAPE̊蓖āB
      */
     private void bindCopyCellWithEscape() {
         String key = COPY_CELL_WITH_ESCAPE;
@@ -130,6 +324,20 @@ final class ResultSetTableAction {
         imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_C, mask), key);
     }
 
+    /**
+     * COPY_COLUMN_NAMES̊蓖āB
+     */
+    private void bindCopyColumnNames() {
+        String key = COPY_COLUMN_NAMES;
+        amap.put(key, new AbstractAction() {
+
+            public void actionPerformed(ActionEvent e) {
+                copyColumnNames();
+            }
+
+        });
+    }
+
     /**
      * PASTE_TO_SELECTED_CELLS̊蓖āB
      */
@@ -157,6 +365,7 @@ final class ResultSetTableAction {
             }
 
         });
+        imap.put(getKeyStroke(KeyEvent.VK_DELETE, false), key);
         imap.put(getKeyStroke(KeyEvent.VK_R, true), key);
     }
 
@@ -241,9 +450,7 @@ final class ResultSetTableAction {
      * @return KeyStroke
      */
     private static KeyStroke getKeyStroke(int key, boolean withControlKey) {
-        return KeyStroke.getKeyStroke(key, (withControlKey
-                ? InputEvent.CTRL_DOWN_MASK
-                : 0));
+        return KeyStroke.getKeyStroke(key, (withControlKey ? InputEvent.CTRL_DOWN_MASK : 0));
     }
 
     /**
@@ -286,8 +493,7 @@ final class ResultSetTableAction {
                 out.print(contents);
                 log.trace(buffer);
             }
-            Clipboard clipboard = Toolkit.getDefaultToolkit()
-                                         .getSystemClipboard();
+            Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
             StringSelection ss = new StringSelection(contents);
             clipboard.setContents(ss, ss);
         } catch (IOException ex) {
@@ -295,6 +501,30 @@ final class ResultSetTableAction {
         }
     }
 
+    /**
+     * 񖼂Nbv{[hɃRs[B
+     */
+    void copyColumnNames() {
+        TableColumnModel columnModel = table.getTableHeader().getColumnModel();
+        int columnCount = columnModel.getColumnCount();
+        if (columnCount < 1) {
+            return;
+        }
+        StringBuffer buffer = new StringBuffer();
+        for (int i = 0; i < columnCount; i++) {
+            if (i != 0) {
+                buffer.append('\t');
+            }
+            buffer.append(columnModel.getColumn(i).getHeaderValue());
+        }
+        if (log.isTraceEnabled()) {
+            log.trace("to clipboard:" + buffer);
+        }
+        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
+        StringSelection ss = new StringSelection(buffer.toString());
+        clipboard.setContents(ss, ss);
+    }
+
     /**
      * IZɃNbv{[h̒l\tB
      */
@@ -337,11 +567,7 @@ final class ResultSetTableAction {
                     Object oldValue = table.getValueAt(rowIndex, columnIndex);
                     Object newValue = values[x];
                     if (!isInsertRow) {
-                        model.updateValueChanged(table,
-                                                 rowIndex,
-                                                 columnIndex,
-                                                 oldValue,
-                                                 newValue);
+                        model.updateValueChanged(table, rowIndex, columnIndex, oldValue, newValue);
                     }
                     table.setValueAt(newValue, rowIndex, columnIndex);
                 }
@@ -365,11 +591,7 @@ final class ResultSetTableAction {
                 for (int x = 0; x < columnCount; x++) {
                     int columnIndex = selectedColumns[x];
                     Object oldValue = table.getValueAt(rowIndex, columnIndex);
-                    model.updateValueChanged(table,
-                                             rowIndex,
-                                             columnIndex,
-                                             oldValue,
-                                             null);
+                    model.updateValueChanged(table, rowIndex, columnIndex, oldValue, null);
                     table.setValueAt(null, rowIndex, columnIndex);
                 }
             }
@@ -465,14 +687,20 @@ final class ResultSetTableAction {
     void deleteSelectedRows() {
         ResultSetTableModel model = (ResultSetTableModel)table.getModel();
         int[] selectedRows = table.getSelectedRows();
-        int selectedRowCount = selectedRows.length;
+        final int selectedRowCount = selectedRows.length;
         if (selectedRowCount < 1) {
-            return;
+            throw new IllegalStateException("Why called ResultSetTableAction.deleteSelectedRows() ?");
         }
-        int selectedRow = selectedRows[0];
+        /*
+         * (#236̑Ή)
+         * ォ珇ɍ폜ĂAтɈsl߂B
+         * ̂߃\[gĂKvB
+         */
+        Arrays.sort(selectedRows);
+        int deletedCount = 0;
         for (int i = 0; i < selectedRowCount; i++) {
-            boolean isSelectedInsertRow = model.isInsertRow(selectedRow);
-            if (!isSelectedInsertRow) {
+            final int selectedRow = selectedRows[i] - deletedCount;
+            if (!model.isInsertRow(selectedRow)) {
                 try {
                     model.updateRowDeleted(table, new int[]{selectedRow});
                 } catch (SQLException ex) {
@@ -480,6 +708,7 @@ final class ResultSetTableAction {
                 }
             }
             model.removeRow(selectedRow);
+            ++deletedCount;
         }
     }
 
