@@ -8,7 +8,7 @@ import javax.swing.table.*;
 /**
  * ResultSetTable̍sԍ\TableCellRendererB
  */
-class ResultSetTableRowNumberCellRenderer extends DefaultTableCellRenderer {
+final class ResultSetTableRowNumberCellRenderer extends DefaultTableCellRenderer {
 
     /** Singleton */
     static final ResultSetTableRowNumberCellRenderer INSTANCE = new ResultSetTableRowNumberCellRenderer();
